
;; Try something like this:

(add-to-list 'load-path "~/Projects/pymacs-2.0/extensions")
(pymacs-load "/home/jj/Projects/pymacs-2.0/extensions.pycomplete")
(load "~/Projects/pymacs-2.0/extensions/python-mode.el")
(load "~/Projects/pymacs-2.0/extensions/pycomplete.el")

;; For playing with menu try:

(pymacs-load "/home/jj/Projects/pymacs-2.0/extensions.menudemo")
